@@ -25,8 +25,8 @@ import siputils
 
 
 # Initialise the globals.
-sip_version = 0x040c01
-sip_version_str = "4.12.1"
+sip_version = 0x040c03
+sip_version_str = "4.12.3"
 py_version = sys.hexversion >> 8
 plat_py_site_dir = None
 plat_py_inc_dir = None
@@ -223,6 +223,13 @@ def patch_files():
         ("siplib", "siplib.sbf")
     )
 
+    # The siplib directory may not exist if we are building away from the
+    # source directory.
+    try:
+        os.mkdir("siplib")
+    except OSError:
+        pass
+
     for f in patched:
         dst_fn = os.path.join(*f)
         src_fn = os.path.join(src_dir, dst_fn + ".in")
